# Copyright 2024- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import api, fields, models


class CompanyShareLine(models.Model):
    _name = "company.share.line"
    _description = "Company share line"
    _check_company_auto = True
    _rec_name = "partner_id"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    share_number = fields.Integer(string="Number of Share", required=True)
    share_unit_price = fields.Monetary(
        string="Share price",
        currency_field="company_currency_id",
        default=lambda self: self.env.company.share_unit_price,
    )
    share_action = fields.Selection(
        [
            ("purchase", "Purchase"),
            ("refund", "Refund"),
            ("guarantee_call", "Appel en garantie"),
            ("storing", "Mise en réserve"),
        ],
        required=True,
        default="purchase",
    )
    share_total_amount = fields.Monetary(
        compute="_compute_share_total_amount",
        store=True,
        currency_field="company_currency_id",
    )
    payment_date = fields.Date()
    partner_id = fields.Many2one(
        comodel_name="res.partner",
        string="Contact",
        required=True,
    )
    company_id = fields.Many2one(
        comodel_name="res.company",
        string="Company",
        required=True,
        change_default=True,
        readonly=True,
        default=lambda self: self.env.company,
    )
    company_currency_id = fields.Many2one(
        comodel_name="res.currency",
        string="Company Currency",
        related="company_id.currency_id",
        readonly=True,
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------
    _sql_constraints = [
        (
            "strictly_positive_share_number",
            "CHECK(share_number>0)",
            "Share count must be strictly positive",
        ),
    ]

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("share_number", "share_unit_price")
    def _compute_share_total_amount(self):
        for share_line in self:
            ratio = 1 if share_line.share_action == "purchase" else -1
            share_line.share_total_amount = (
                ratio * share_line.share_unit_price * share_line.share_number
            )

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
